#!/bin/sh

############################################################################
#
# Copyright (C) 2004 by Marc Ballarin
# Ballarin.Marc@gmx.de
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
############################################################################

#Version: 0.2

#TODO:	- improve download reliability
#		?- support multiple upgrade steps at once (like 3.2.0 -> 3.2.2)
#		- support download of complete files if no old version exists
#		?- allow retries after wrong input

# Temporary workingdir
WD=`mktemp -d -q` || (echo "Could not create temporary directory!"; exit)

# Path containing deltas (or where they should be stored)
DELTAPATH=$WD

# Path to old sources
OLDPATH=""

# Destination of patched sources
NEWPATH=""

ERRORLIST=""

SERVER="ftp://ftp.kde.org/pub/kde"

# Files not available as old version
MISSING=""

# Old version of KDE (default)
OV=3.2.1
# Target version (default)
NV=3.2.2
#Versions of arts
ANV=""
AOV=""

# files in KDE distribution
FILELIST="arts		\
	kdelibs			\
	kdebase			\
	kdemultimedia	\
	kdeadmin		\
	kdeartwork		\
	kdegraphics		\
	kdenetwork		\
	kdeaccessibility\
	kdesdk			\
	kdepim			\
	kdetoys			\
	kdeutils		\
	kdegames		\
	kdeedu			\
	kdebindings		\
	kdeaddons		\
	quanta"


clean_up(){
	cd $OLDPWD
	rm -rf $WD
}
	
user_abort(){
	clean_up
	kdialog --msgbox "Cancelled on user request!"
	exit 1
}

# $1: error msg
error_abort(){
	clean_up
 	kdialog --error "$1"
	exit 2
}

# Check which files are actually there
# $1: path to check; $2: list of files
check_files(){
	TEMP=""
	for i in $2
	do
		if [ -e "$1/$i-$OV.tar.bz2" ] || [ -e "$1/$i-$AOV.tar.bz2" ]; then
			TEMP="$TEMP $i"
		else
			MISSING="$MISSING $i"
		fi
	done
	if [ "$TEMP" = "" ];then
		error_abort "No old KDE sources found in $1 !"
	fi
	echo $TEMP
}

# Choose which files should really be updated
# $1: list of files
choose_files(){
	CMD=""
	for i in $1
	do
		CMD="$CMD $i $i on"
	done
	TEMP=`kdialog --checklist "Which files should be updated?" \
	--title "Choose files" $CMD` || user_abort
	# kdialog quotes all strings - remove those quotes as they confuse wget
	echo $TEMP | sed s/\"/\/g
}

# TODO: - Download a server list and allow selection
choose_server(){
	SERVER=`kdialog --inputbox "Enter URL to files. See http://www.kde.org/mirrors/ftp.php:" \
	--title "Choose server" "$SERVER"` || user_abort	
	echo "$SERVER/stable/$NV/src"
}

# $1: server to use; $2 filelist
fetch_deltas(){
	for i in $2
	do
		if [ $i = "arts" ];then
			wget --read-timeout=240 --wait=5 -nc "$1/$i-$AOV-$ANV.tar.xdelta"|| \
			error_abort "Could not download $i. Please make sure URL is correct, and mirror is up to date."
		else
			wget --read-timeout=240 --wait=5 -nc "$1/$i-$OV-$NV.tar.xdelta"|| \
			error_abort "Could not download $i. Please make sure URL is correct, and mirror is up to date."
		fi
	done
}

get_versions(){
	VALID=0
	while [ $VALID = 0 ]
	do
		OV=`kdialog --inputbox "Please enter the version number of current sourcefiles" $OV` || user_abort
		NV=`kdialog --inputbox "To which version do you want to update?" $NV` || user_abort
		VALID=1
	done
	# Try to guess the right version for arts
	# Simply set major version to 1
	ANV="1.${NV:2:4}"
	AOV="1.${OV:2:4}"
}

# Update the files
#  1. Uncompress old file from $1 to workdir
#  2. Apply delta to uncompressed file
#  3. Recompress file to $2
#  4. Remove delta and uncompressed files
# $1: path to old sources; $2: path to new sources; $3: path to deltas; $4: list of files
do_updates(){
for FILE in $4
do
	if [ $FILE = "arts" ];then
		_OV=$AOV; _NV=$ANV;
	else
		_OV=$OV; _NV=$NV;
	fi
   (echo Extracting $FILE-$_OV.tar.bz2; bzip2 -cd $1/$FILE-$_OV.tar.bz2 > $WD/$FILE-$_OV.tar) &&\
   (echo Patching $FILE-$_OV.tar; xdelta patch $3/$FILE-$_OV-$_NV.tar.xdelta) &&\
   (echo Compressing $FILE-$_NV.tar; bzip2 -cz $WD/$FILE-$_NV.tar > $2/$FILE-$_NV.tar.bz2) &&\
   (rm -f $3/$FILE-$_OV-$_NV.tar.xdelta; rm -f $WD/$FILE-*.tar)||\
   ERRORLIST="$ERRORLIST $FILE could not be patched; "
done
}

# $1: starting dir; $2: message
get_path(){
	TEMP=`kdialog --getexistingdirectory $1 --title\
	"$2"` || user_abort
	echo $TEMP
}


# $1 list of files
ask_for_deltas(){
	kdialog  --yesnocancel "Do you already have the patch files?"

	A=$?
	case $A in
		0) DELTAPATH=`get_path "$HOME" "Which directory contains the deltas?"` || user_abort
		;;
		1) SERVER=`choose_server`||exit $?; fetch_deltas "$SERVER" "$1"
		;;
		2) user_abort
		;;
	esac
}

# $1: list of errors
print_log(){
	if [ "$1" = "" ]; then
		echo $2
	else
		echo $3 $1
	fi
}

# end of functions

# Cleanup, even when ^C or kill is used
trap 'clean_up; exit 1' 1 2 3 15

cd $WD

get_versions 

OLDPATH=`get_path "$HOME" "Which directory contains the old KDE sources?"`||exit $?

FILELIST=`check_files "$OLDPATH" "$FILELIST"`||exit $?

FILELIST=`choose_files "$FILELIST"`||exit $?

ask_for_deltas "$FILELIST" ||exit $?

NEWPATH=`get_path "$OLDPATH" "Where should the new files be placed?"`||exit $?

do_updates "$OLDPATH" "$NEWPATH" "$DELTAPATH" "$FILELIST"

print_log "$ERRORLIST" "Succesfully completed." "Possible problems: "

print_log "$MISSING" "" "The old versions for the following packages was not found: "

clean_up
